unit FILAC_Config;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls, NumEdits, Vcl.ExtCtrls;

type
  TConfigDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    ApplyButton: TButton;
    WarningLbl: TLabel;
    OptionPage: TPageControl;
    WindowTab: TTabSheet;
    CollapsedGroup: TGroupBox;
    SizeLbl: TLabel;
    SizeEdt: TUpDownEdit;
    TransLbl: TLabel;
    TransEdt: TUpDownEdit;
    ExpandedGroup: TGroupBox;
    WidthLbl: TLabel;
    WidthEdt: TUpDownEdit;
    HeightLbl: TLabel;
    HeightEdt: TUpDownEdit;
    ColoringGroup: TGroupBox;
    BaseLbl: TLabel;
    BaseBox: TColorBox;
    ShiftChk: TCheckBox;
    FrameTab: TTabSheet;
    DefaultStylesGroup: TGroupBox;
    HasLinesChk: TCheckBox;
    FullRowSelectChk: TCheckBox;
    SpringExpandChk: TCheckBox;
    HorizontalScrollChk: TCheckBox;
    ShowSelectionAlwaysChk: TCheckBox;
    NoEditLabelsChk: TCheckBox;
    AutoHScrollChk: TCheckBox;
    FadeInOutExpandosChk: TCheckBox;
    TreeTab: TTabSheet;
    DefaultFiltersGroup: TGroupBox;
    FoldersChk: TCheckBox;
    NonFoldersChk: TCheckBox;
    IncludeHiddenChk: TCheckBox;
    IncludeSuperHiddenChk: TCheckBox;
    CompletionTab: TTabSheet;
    MaintainGroup: TGroupBox;
    FromLbl: TLabel;
    FromCbx: TComboBox;
    FrameLbl: TLabel;
    FrameCbx: TComboBox;
    TreeLbl: TLabel;
    TreeCbx: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure SizeEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure TransEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure WidthEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure HeightEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
      Relation: TNumericallyRelationship);
    procedure BaseBoxChange(Sender: TObject);
    procedure BaseBoxDropDown(Sender: TObject);
    procedure ShiftChkClick(Sender: TObject);
    procedure DefaultStyleChksClick(Sender: TObject);
    procedure DefaultFilterChksClick(Sender: TObject);
    procedure MaintainCbxsChange(Sender: TObject);
  private
    { Private 錾 }
    FUpdateCount: Integer;
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure UpdateValues;
    procedure SetValues;
    procedure ResetValues;
  public
    { Public 錾 }
  end;

var
  ConfigDialog: TConfigDialog;

implementation

{$R *.dfm}

uses
  FILAC_Main,
  VistaShellCtrls;

resourcestring
  SSize = '';
  STrans = 'x';
  SWidth = '';
  SHeight = '';
  SDisaccord = '%s͈͊Oł';
  SNonNumber = 's%sł';

procedure TConfigDialog.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TConfigDialog.EndUpdate;
begin
  Dec(FUpdateCount);
  if FUpdateCount = 0 then
  begin
    WarningLbl.Caption := '';

    case SizeEdt.Relation of
      nrDisaccord: WarningLbl.Caption := Format(SDisaccord, [SSize]);
      nrNonNumber: WarningLbl.Caption := Format(SNonNumber, [SSize]);
    end;
    case TransEdt.Relation of
      nrDisaccord: WarningLbl.Caption := Format(SDisaccord, [STrans]);
      nrNonNumber: WarningLbl.Caption := Format(SNonNumber, [STrans]);
    end;
    case WidthEdt.Relation of
      nrDisaccord: WarningLbl.Caption := Format(SDisaccord, [SWidth]);
      nrNonNumber: WarningLbl.Caption := Format(SNonNumber, [SWidth]);
    end;
    case HeightEdt.Relation of
      nrDisaccord: WarningLbl.Caption := Format(SDisaccord, [SHeight]);
      nrNonNumber: WarningLbl.Caption := Format(SNonNumber, [SHeight]);
    end;

    OKButton.Enabled := WarningLbl.Caption = '';
    ApplyButton.Enabled := OKButton.Enabled;
  end;
end;

procedure TConfigDialog.UpdateValues;
var
  Styles: TNamespaceTreeControlStyles;
  Filters: TShellContactFilters;
  Maintains: TChangeParts;
begin
  BeginUpdate;

  //Window
  SizeEdt.Value := SizeEdt.Tag;
  TransEdt.Value := TransEdt.Tag;
  WidthEdt.Value := WidthEdt.Tag;
  HeightEdt.Value := HeightEdt.Tag;
  BaseBox.Selected := BaseBox.Tag;
  ShiftChk.Checked := ShiftChk.Tag <> 0;
  //Frame
  Styles := EnumNTCS(DefaultStylesGroup.Tag);
  HasLinesChk.Checked := nstcsHasLines in Styles;
  FullRowSelectChk.Checked := nstcsFullRowSelect in Styles;
  SpringExpandChk.Checked := nstcsSpringExpand in Styles;
  HorizontalScrollChk.Checked := nstcsHorizontalScroll in Styles;
  ShowSelectionAlwaysChk.Checked := nstcsShowSelectionAlways in Styles;
  NoEditLabelsChk.Checked := nstcsNoEditLabels in Styles;
  AutoHScrollChk.Checked := nstcsAutoHScroll in Styles;
  FadeInOutExpandosChk.Checked := nstcsFadeInOutExpandos in Styles;
  //Root
  Filters := EnumSCF(DefaultFiltersGroup.Tag);
  FoldersChk.Checked := shcontfFolders in Filters;
  NonFoldersChk.Checked := shcontfNonFolders in Filters;
  IncludeHiddenChk.Checked := shcontfIncludeHidden in Filters;
  IncludeSuperHiddenChk.Checked := shcontfIncludeSuperHidden in Filters;
  //Completion
  Maintains := EnumCP(MaintainGroup.Tag);
  if cpForm in Maintains then
    FromCbx.ItemIndex := 0
  else
    FromCbx.ItemIndex := 2;
  if cpFrame in Maintains then
    FrameCbx.ItemIndex := 0
  else
    FrameCbx.ItemIndex := 2;
  if cpTree in Maintains then
    TreeCbx.ItemIndex := 0
  else
    TreeCbx.ItemIndex := 2;

  EndUpdate;
end;

procedure TConfigDialog.SetValues;
begin
  //Window
  SizeEdt.Tag := MainForm.CollapseSize;
  TransEdt.Tag := MainForm.AlphaBlendValue;
  WidthEdt.Tag := MainForm.ExpandWidth;
  HeightEdt.Tag := MainForm.ExpandHeight;
  BaseBox.Tag := MainForm.BaseColor;
  ShiftChk.Tag := Ord(MainForm.ColorShift);
  //Frame
  DefaultStylesGroup.Tag := NumNTCS(MainForm.DefaultStyles);
  //Root
  DefaultFiltersGroup.Tag := NumSCF(MainForm.DefaultFilters);
  //Completion
  MaintainGroup.Tag := NumCP(MainForm.Maintains);

  UpdateValues;
end;

procedure TConfigDialog.ResetValues;
begin
  //Window
  MainForm.CollapseSize := SizeEdt.Tag;
  MainForm.AlphaBlendValue := TransEdt.Tag;
  MainForm.ExpandWidth := WidthEdt.Tag;
  MainForm.ExpandWidth := HeightEdt.Tag;
  MainForm.BaseColor := BaseBox.Tag;
  MainForm.ColorShift := ShiftChk.Tag <> 0;
  //Frame
  MainForm.DefaultStyles := EnumNTCS(DefaultStylesGroup.Tag);
  //Root
  MainForm.DefaultFilters := EnumSCF(DefaultFiltersGroup.Tag);
  //Completion
  MainForm.Maintains := EnumCP(MaintainGroup.Tag);

  UpdateValues;
end;

procedure TConfigDialog.FormCreate(Sender: TObject);
begin
  {$IFDEF RELEASE}
  OptionPage.TabIndex := 0;
  {$ENDIF RELEASE}
end;

procedure TConfigDialog.FormShow(Sender: TObject);
begin
  FUpdateCount := 0;

  SetValues;
end;

procedure TConfigDialog.OKButtonClick(Sender: TObject);
begin
  //
end;

procedure TConfigDialog.CancelButtonClick(Sender: TObject);
begin
  if OKButton.Enabled then
    ResetValues;
end;

procedure TConfigDialog.ApplyButtonClick(Sender: TObject);
begin
  MainForm.Maintains := EnumCP(MaintainGroup.Tag);
  //! SetValues;
  Close;
end;

procedure TConfigDialog.SizeEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  if Application.ActiveFormHandle = Handle then
  begin
    BeginUpdate;

    if Relation = nrAccord then
      MainForm.CollapseSize := APosition;

    EndUpdate;
  end;
end;

procedure TConfigDialog.TransEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  if Application.ActiveFormHandle = Handle then
  begin
    BeginUpdate;

    if Relation = nrAccord then
      MainForm.AlphaBlendValue := APosition;

    EndUpdate;
  end;
end;

procedure TConfigDialog.WidthEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  if Application.ActiveFormHandle = Handle then
  begin
    BeginUpdate;

    if Relation = nrAccord then
      MainForm.ExpandWidth := APosition;

    EndUpdate;
  end;
end;

procedure TConfigDialog.HeightEdtVerify(Sender: TObject; var AText: string; var APosition: Integer;
  Relation: TNumericallyRelationship);
begin
  if Application.ActiveFormHandle = Handle then
  begin
    BeginUpdate;

    if Relation = nrAccord then
      MainForm.ExpandHeight := APosition;

    EndUpdate;
  end;
end;

procedure TConfigDialog.BaseBoxChange(Sender: TObject);
begin
  BeginUpdate;

  with BaseBox do
  begin
    MainForm.BaseColor := Selected;
    if Width < Canvas.TextWidth(Items.Strings[ItemIndex]) then
      Hint := Items.Strings[ItemIndex]
    else
      Hint := '';
  end;

  EndUpdate;
end;

procedure TConfigDialog.BaseBoxDropDown(Sender: TObject);
var
  i, ListWidth, ItemWidth: Integer;
begin
  with BaseBox do
  begin
    ListWidth := Width;
    for i := 0 to Items.Count - 1 do
    begin
      ItemWidth := Canvas.TextWidth(Items.Strings[i]) + 50;
      if ListWidth < ItemWidth then
        ListWidth := ItemWidth;
    end;
    Perform(CB_SETDROPPEDWIDTH, ListWidth, 0);
  end;
end;

procedure TConfigDialog.ShiftChkClick(Sender: TObject);
begin
  BeginUpdate;

  MainForm.ColorShift := ShiftChk.Checked;

  EndUpdate;
end;

procedure TConfigDialog.DefaultStyleChksClick(Sender: TObject);
var
  Style: TNamespaceTreeControlStyle;
  NewStyles: TNamespaceTreeControlStyles;
begin
  BeginUpdate;

  with TCheckBox(Sender) do
  begin
    Style := TNamespaceTreeControlStyle(Tag);
    if Checked then
      case Style of
        nstcsHasLines: FullRowSelectChk.Checked := False;
        nstcsFullRowSelect: HasLinesChk.Checked := False;
      end;

    NewStyles := MainForm.DefaultStyles;
    if Checked then
      Include(NewStyles, Style)
    else
      Exclude(NewStyles, Style);
    MainForm.DefaultStyles := NewStyles;
  end;

  EndUpdate;
end;

procedure TConfigDialog.DefaultFilterChksClick(Sender: TObject);
var
  NewFilters: TShellContactFilters;
  Filter: TShellContactFilter;
begin
  BeginUpdate;

  NewFilters := MainForm.DefaultFilters;
  with TCheckBox(Sender) do
  begin
    Filter := TShellContactFilter(Tag);
    if Checked then
      Include(NewFilters, Filter)
    else
      Exclude(NewFilters, Filter);
  end;
  MainForm.DefaultFilters := NewFilters;

  EndUpdate;
end;

procedure TConfigDialog.MaintainCbxsChange(Sender: TObject);
var
  NewParts: TChangeParts;
  Part: TChangePart;
begin
  BeginUpdate;

  NewParts := MainForm.Maintains;
  with TComboBox(Sender) do
  begin
    Part := TChangePart(Tag);
    if ItemIndex = 0 then
      Include(NewParts, Part)
    else
      Exclude(NewParts, Part);
  end;
  MainForm.Maintains := NewParts;

  EndUpdate;
end;

end.
